package simulation;

import java.util.Random;

public class Simulation {

    private static final Random RNG = new Random();

    public static int[][] generateOutLinks(int n) {
        int[][] result = new int[n][n];

        for (int i = 0; i < result.length; i++) {
            for (int j = 0; j < result.length; j++) {
                result[i][j] = RNG.nextInt(8);
            } // for
        } // for

        return result;
    } // generateOutLinks( int )

    public static int[] sumOutLinks(int[][] table) {
        int[] result = new int[table.length];

        for (int i = 0; i < table.length; i++) {
            result[i] = 0;
            for (int j = 0; j < table.length; j++) {
                result[i] += table[i][j];
            } // for
        } // for

        return result;
    } // sumOutLinks( int [][] )

    public static double[][] pdf(int[][] table, int[] totals) {
        int n = table.length;
        double[][] result = new double[n][n];

        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                result[i][j] = table[i][j] / ((double) totals[i]);
            } // for
        } // for

        return result;
    } // pdf( int [][], int [] )

    public static double[][] cdf(double[][] pdf) {
        int n = pdf.length;
        double[][] result = new double[n][n];

        for (int i = 0; i < n; i++) {
            result[i][0] = pdf[i][0];
            for (int j = 1; j < n; j++) {
                result[i][j] = result[i][j - 1] + pdf[i][j];
            } // for
        } // for

        return result;
    } // cdf( double [][] )

    public static int nextPage(int currentPage, double[][] cdf) {
        int result = 0;
        double r = RNG.nextDouble();
        
        while( r > cdf[currentPage][result] ) {
            result++;
        } // while
        
        return result;
    } // nextPage( int, double [][] )

    public static void printArray(int[][] table, int[] rowSums) {
        for (int i = 0; i < table.length; i++) {
            for (int j = 0; j < table.length; j++) {
                System.out.printf("%4d ", table[i][j]);
            } // for
            System.out.printf(" | %4d ", rowSums[i]);
            System.out.println();
        } // for
    } // printArray( int [][], int [] )

    public static void printDF(double[][] table) {
        for (int i = 0; i < table.length; i++) {
            for (int j = 0; j < table.length; j++) {
                System.out.printf("%8.4f ", table[i][j]);
            } // for
            System.out.println();
        } // for
    } // printArray( int [][] )

    public static void main(String[] args) {
        int[][] table = generateOutLinks(4);
        int[] rowSums = sumOutLinks(table);
        printArray(table, rowSums);
        double[][] pdf = pdf(table, rowSums);

        System.out.println();

        printDF(pdf);

        double[][] cdf = cdf(pdf);

        System.out.println();

        printDF(cdf);
        
        int currentPage = 0;
        for( int i = 0; i < 100; i++ ) {
            System.out.println( nextPage( currentPage, cdf ));;
        } // for
    } // main( String [] )

} // Simulation
